package org.apache.maven.linkcheck.validation;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache Maven" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache Maven", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * ====================================================================
 */
 
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author <a href="mailto:bwalding@apache.org">Ben Walding</a>
 * @version $Id: LinkValidatorManager.java,v 1.4 2003/02/07 12:01:04 evenisse Exp $
 */

public class LinkValidatorManager {
    /**
     * Log for debug output
     */
    private static Log LOG = LogFactory.getLog(LinkValidatorManager.class);

    private List validators = new ArrayList();
    private LinkValidatorCache cache = new LinkValidatorCache(this);
    private String exclude;

    public LinkValidatorManager() {

    }

    public void addLinkValidator(LinkValidator lv) {
        validators.add(lv);
    }

    public List getValidators() {
        return validators;
    }

    public LinkValidationResult validateLink(LinkValidationItem lvi)
        throws Exception {
        {
            LinkValidationResult status = cache.getCachedResult(lvi);
            if (status != null) {
                return status;
            }
        }

        if (exclude != null && lvi.getLink().startsWith(exclude)) {
            return new LinkValidationResult(LinkValidationResult.VALID, false);
        }

        Iterator iter = validators.iterator();
        while (iter.hasNext()) {
            LinkValidator lv = (LinkValidator) iter.next();

            Object resourceKey = lv.getResourceKey(lvi);

            if (resourceKey != null) {

                LinkValidationResult lvr = lv.validateLink(lvi);

                if (lvr.getStatus() == LinkValidationResult.NOTMINE) {
                    continue;
                }

                cache.setCachedResult(resourceKey, lvr);
                return lvr;

            }

        }

        LOG.info("Unable to validate link : " + lvi.getLink());
        return new LinkValidationResult(LinkValidationResult.UNKNOWN, false);
    }

    public void loadCache(String cacheFilename) {
        try {
            File f = new File(cacheFilename);
            if (f.exists()) {
                this.cache.load(new FileInputStream(cacheFilename));
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    public void saveCache(String cacheFilename) {
        try {
            File cacheFile = new File(cacheFilename);
            File dir = cacheFile.getParentFile();
            if (dir != null) {
                dir.mkdirs();
            }
            this.cache.save(new FileOutputStream(cacheFilename));
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    /**
     * Returns the exclude.
     * @return String
     */
    public String getExclude() {
        return exclude;
    }

    /**
     * Sets the exclude.
     * @param exclude The exclude to set
     */
    public void setExclude(String exclude) {
        this.exclude = exclude;
    }

}
